import { useState } from "react";
import PropTypes from "prop-types";
import "../css/elements/File.css";

const File = ({ icon, fileName, initialX, initialY, link, onMove }) => {
  const [position, setPosition] = useState({ x: initialX, y: initialY });
  const [isDragging, setIsDragging] = useState(false);
  const [offset, setOffset] = useState({ x: 0, y: 0 });

  const handleMouseDown = (e) => {
    e.preventDefault();
    setIsDragging(true);
    setOffset({
      x: e.clientX - position.x,
      y: e.clientY - position.y,
    });
  };

  const handleMouseMove = (e) => {
    if (!isDragging) return;

    const newX = e.clientX - offset.x;
    const newY = e.clientY - offset.y;

    // Przeciąganie tylko w obrębie okna folderu
    const folderRect = document
      .querySelector(".folder-window")
      ?.getBoundingClientRect();
    const fileWidth = 100;
    const fileHeight = 100;

    if (folderRect) {
      const minX = 0;
      const minY = 0;
      const maxX = folderRect.width - fileWidth;
      const maxY = folderRect.height - fileHeight;

      // Ograniczenie przeciągania pliku w obrębie okna
      const boundedX = Math.max(minX, Math.min(newX, maxX));
      const boundedY = Math.max(minY, Math.min(newY, maxY));

      setPosition({ x: boundedX, y: boundedY });
    } else {
      // Przeciąganie bez ograniczeń
      setPosition({ x: newX, y: newY });
    }
  };

  const handleMouseUp = () => {
    setIsDragging(false);
    if (onMove) onMove(position);
  };

  return (
    <div
      className="file"
      style={{
        left: `${position.x}px`,
        top: `${position.y}px`,
        userSelect: "none",
      }}
      onMouseDown={handleMouseDown}
      onMouseMove={handleMouseMove}
      onMouseUp={handleMouseUp}
      onDoubleClick={() => window.open(link, "_blank")}
    >
      <img
        className="file-icon-img"
        src={icon}
        alt="File Icon"
        draggable="false"
      />
      <span className="file-name">{fileName}</span>
    </div>
  );
};

File.propTypes = {
  icon: PropTypes.string.isRequired,
  fileName: PropTypes.string.isRequired,
  link: PropTypes.string.isRequired,
  initialX: PropTypes.number.isRequired,
  initialY: PropTypes.number.isRequired,
  onMove: PropTypes.func,
};

export default File;
