import { useState } from "react";
import PropTypes from "prop-types";
import File from "./File";
import "../css/elements/Folder.css";

const Folder = ({ icon, folderName, initialX, initialY, files }) => {
  const [position, setPosition] = useState({ x: initialX, y: initialY });
  const [isDragging, setIsDragging] = useState(false);
  const [offset, setOffset] = useState({ x: 0, y: 0 });
  const [isOpen, setIsOpen] = useState(false);
  const [windowPosition, setWindowPosition] = useState({ x: 100, y: 100 });
  const [isWindowDragging, setIsWindowDragging] = useState(false);
  const [windowOffset, setWindowOffset] = useState({ x: 0, y: 0 });

  const handleMouseDown = (e) => {
    e.preventDefault();
    setIsDragging(true);
    setOffset({
      x: e.clientX - position.x,
      y: e.clientY - position.y,
    });
  };

  const handleMouseMove = (e) => {
    if (isDragging) {
      const newX = e.clientX - offset.x;
      const newY = e.clientY - offset.y;
      setPosition({ x: newX, y: newY });
    } else if (isWindowDragging) {
      const newX = e.clientX - windowOffset.x;
      const newY = e.clientY - windowOffset.y;
      setWindowPosition({ x: newX, y: newY });
    }
  };

  const handleMouseUp = () => {
    setIsDragging(false);
    setIsWindowDragging(false);
  };

  const handleWindowMouseDown = (e) => {
    e.preventDefault();
    setIsWindowDragging(true);
    setWindowOffset({
      x: e.clientX - windowPosition.x,
      y: e.clientY - windowPosition.y,
    });
  };

  const handleCloseFolder = () => {
    setIsOpen(false);
  };

  return (
    <>
      <div
        className="file"
        style={{ left: `${position.x}px`, top: `${position.y}px` }}
        onMouseDown={handleMouseDown}
        onMouseMove={handleMouseMove}
        onMouseUp={handleMouseUp}
        onDoubleClick={() => setIsOpen(true)}
      >
        <img src={icon} alt="Folder Icon" className="file-icon" />
        <span className="file-name">{folderName}</span>
      </div>

      {isOpen && (
        <div
          className="folder-window"
          style={{
            left: `${windowPosition.x}px`,
            top: `${windowPosition.y}px`,
          }}
          onMouseMove={handleMouseMove}
          onMouseUp={handleMouseUp}
        >
          <div className="folder-header" onMouseDown={handleWindowMouseDown}>
            <span>{folderName}</span>
            <button onClick={handleCloseFolder}>X</button>
          </div>
          <div className="folder-content">
            {files.map((file, index) => (
              <File
                key={index}
                icon={file.icon}
                fileName={file.fileName}
                link={file.link}
                initialX={50 + index * 100}
                initialY={50}
                onMouseDown={(e) => e.stopPropagation()}
              />
            ))}
          </div>
        </div>
      )}
    </>
  );
};

Folder.propTypes = {
  icon: PropTypes.string.isRequired,
  folderName: PropTypes.string.isRequired,
  initialX: PropTypes.number.isRequired,
  initialY: PropTypes.number.isRequired,
  files: PropTypes.array.isRequired,
};

export default Folder;
